package ASCOPE::Email::Graph;
use strict;

# $Id: Graph.pm,v 1.3 2004/06/07 14:22:18 asc Exp $

$ASCOPE::Email::Graph::VERSION = '0.1';

=head1 NAME

ASCOPE::Email::Graph - Aaron's widget for graphing email correspondence.

=head1 SYNOPSIS

  my $graph = ASCOPE::Email::Graph->new();

  $graph->add_mbox("/path/to/mbox");
  $graph->as_svg("/path/to/graph.svg");

=head1 DESCRIPTION

Aaron's widget for graphing email correspondence.

=cut

use Email::Folder;
use Email::Find;

use Color::Calc;

use GraphViz;
use Digest::SHA1 qw(sha1_hex);

sub import {
    my $self = shift;

    my @valid = qw(as_canon
		   as_text
		   as_ps
		   as_hpgl
		   as_pcl
		   as_mif
		   as_pic
		   as_gd
		   as_gd2
		   as_jpeg
		   as_png
		   as_wbmp
		   as_cmap
		   as_ismap
		   as_imap
		   as_vrml
		   as_vtx
		   as_mp
		   as_fig
		   as_svg
		   as_plain);

    foreach my $meth (@valid) {
	no strict "refs";

	*$meth = sub {
	    my $self    = shift;
	    my $outfile = shift;

	    my $gv = $self->_graph();
	    $gv->$meth($outfile);
	};
    }

    return 1;
}

=head1 PACKAGE METHODS

=cut

=head2 __PACKAGE__->new(\%args)

=over 4

=item * B<colour>

String; a valid X11 color name or a RRGGBB hex string.

Base colour used to fill individual nodes; each node will be filled
using a percentage of this colour based on its weight.

Default is I<grey>

=item * B<color>

No comment.

=item * B<fontsize>

Int.

Font size used to write email addresses.

Default is I<6>

=item * B<hash>

Boolean.

Hash email addresses using I<Digest::SHA1>.

Default is I<false>.

=back

Returns a I<ASCOPE::Email::Graph> object. Woot!

=cut

sub new {
    my $pkg  = shift;
    my $args = shift;

    my $self = {"_colour"   => ($args->{colour}   || $args->{color} || "grey"),
		"_fontsize" => ($args->{fontsize} || '6'),
		"_hash"     => ($args->{foafify}  || 0),
		"_nodes"    => {},
		"_rels"     => {}};

    return bless $self,$pkg;
}

=head1 OBJECT METHODS

=cut

=head2 $obj->add_mbox($path)

=cut

sub add_mbox {
    my $self = shift;
    my $path = shift;

    my $mbox = Email::Folder->new($path);

    if (! $mbox) {
	warn "Failed to create mbox for $path : $!\n";
	return 0;
    }
    
    while (my $msg = $mbox->next_message()) {
	$self->_address($msg);
    }
    
    return 1;
}

=head2 $obj->as_canon($path);

=head2 $obj->as_text($path);

=head2 $obj->as_ps($path);

=head2 $obj->as_hpgl($path);

=head2 $obj->as_pcl($path);

=head2 $obj->as_mif($path);

=head2 $obj->as_pic($path);

=head2 $obj->as_gd($path);

=head2 $obj->as_gd2($path);

=head2 $obj->as_jpeg($path);

=head2 $obj->as_png($path);

=head2 $obj->as_wbmp($path);

=head2 $obj->as_cmap($path);

=head2 $obj->as_ismap($path);

=head2 $obj->as_imap($path);

=head2 $obj->as_vrml($path);

=head2 $obj->as_vtx($path);

=head2 $obj->as_mp($path);

=head2 $obj->as_fig($path);

=head2 $obj->as_svg($path);

=head2 $obj->as_plain($path);

=cut

sub _address {
    my $self = shift;
    my $msg  = shift;

    my %addr = (from => [],
		to   => [],
		cc   => []);

    #

    foreach my $type ("from","to","cc") {
	
	foreach my $string ($msg->header($type)) {
	    
	    &find_emails(\$string, sub {
		my ($email, $orig_email) = @_;
		my $address = $email->format();
	
		if ($self->{'_hash'}) {
		    $address = sha1_hex($address);
		}

		push @{$addr{$type}}, $address;
		$self->{'_nodes'}->{ $address } ++;
	    });
	}
    }
    
    #

    foreach my $from (@{$addr{"from"}}) {
	
	foreach my $to (@{$addr{"to"}}) {
	    $self->{'_rels'}->{ $from }->{ $to } ++;
	}
	
	foreach my $cc (@{$addr{"cc"}}) {
	    $self->{'_rels'}->{ $from }->{ $cc } ++;
	}
    }
    
    #
    
    return 1;
}

sub _graph {
    my $self = shift;
  
    my $g = GraphViz->new(directed    => 1,
			  concentrate => 1);

    my $first = (sort {$b <=> $a } values %{$self->{'_nodes'}})[0];

    #

    foreach my $who (keys %{$self->{'_nodes'}}) {
	
       	my $colour = $self->{'_colour'};
	my $alpha  = undef;

	my $weight = $self->{'_nodes'}->{$who};

	if ($weight != $first) {
	    $alpha  = 1 - ($weight / $first);
	    $colour = sprintf("#%s",color_light_hex($colour,
						    $alpha));
	}

	# print "[$who] $colour ($weight) ($alpha)\n";

	$g->add_node($who,
		     fontsize  => $self->{'_fontsize'},
		     style     => "filled",
		     fillcolor => $colour);
    }

    #

    foreach my $from (keys %{$self->{'_rels'}}) {
	
	foreach my $to (keys %{$self->{'_rels'}->{$from}}) {
	    
	    $g->add_edge($from    => $to,
			 minlen   => 2,
			 weight   => $self->{'_nodes'}->{$from});
	    
	}
    }
 
    #

    return $g;
}

sub DESTROY {}

=head1 VERSION

0.1

=head1 DATE

$Date: 2004/06/07 14:22:18 $

=head1 AUTHOR

Aaron Straup Cope E<lt>ascope@cpan.orgE<gt>

=head1 LICENSE

Copyright (c) 2004 Aaron Straup Cope. All Rights Reserved.

This is free software, you may use it and distribute it under
the same terms as Perl itself.

=cut

return 1;

__END__

